{% extends "base.html" %}

{% block content %}

{% if not_auth_warn %}
  <div class="alert alert-danger" role="alert">Not authenticated</div>
{% endif %}

{% if success_slo %}
  <div class="alert alert-success" role="alert">Successfully logged out</div>
{% endif %}

{% if paint_logout %}
  {% if attributes %}
    <table class="table table-striped">
      <thead>
        <th>Name</th><th>Values</th>
      </thead>
      <tbody>
        {% for attr in attributes %}
          <tr><td>{{ attr.0 }}</td>
          <td><ul class="list-unstyled">
            {% for val in attr.1 %}
              <li>{{ val }}</li>
            {% endfor %}
          </ul></td></tr>
        {% endfor %}
      </tbody>
    </table>
  {% else %}
    <div class="alert alert-danger" role="alert">You don't have any attributes</div>
  {% endif %}
  <a href="{{ url_for('sso_saml.slo', idp='onelogin') }}" class="btn btn-danger">Logout</a>
{% else %}
  <a href="{{ url_for('sso_saml.sso', idp='onelogin') }}" class="btn btn-primary">Login</a>
  <a href="{{ url_for('sso_saml.sso', idp='onelogin', next='/attrs') }}" class="btn btn-info">Login and access to attrs page</a>
{% endif %}

{% endblock %}
